==> mongodb-ui-components-4.0.2.tgz: package/src/code-widget/index.js <==
const React = require('react')
const Dropdown = require('../dropdown')
const clipboardCopy = require('../shared/clipboard-copy')
const easing = require('../shared/easing')
const style = require('../shared/style')
const script = require('../shared/script')
const widgetStyle = require('./style')

class CodeWidget extends React.PureComponent {
  constructor (props) {
    super(props)

    this.state = {
      activeTab: 0,
      activeStage: 0,
      isClipboardSupported: true,
      copyBtnText: 'Copy Snippet'
    }

    this.minimaps = []
    this.stageTitles = props.descriptions.map((item) => item.title)

    this.onTabSelect = this.onTabSelect.bind(this)
    this.onNavigateStage = this.onNavigateStage.bind(this)
    this.onDropdownSelect = this.onDropdownSelect.bind(this)
    this.onCopyBtnClick = this.onCopyBtnClick.bind(this)
    this.revertCopiedText = this.revertCopiedText.bind(this)
    this.renderTabBtn = this.renderTabBtn.bind(this)
    this.renderTabs = this.renderTabs.bind(this)
  }

  componentDidMount () {
    style('code-widget', widgetStyle())
    script('script', 'https://cdnjs.cloudflare.com/ajax/libs/highlight.js/9.10.0/highlight.min.js')
    script('link', 'https://cdnjs.cloudflare.com/ajax/libs/highlight.js/9.10.0/styles/default.min.css')

    this.minimaps.forEach((minimap, i) => {
      minimap.innerHTML = this.renderMinimap(
        minimap.innerHTML, minimap.className, this.stageTitles.length)
    })

    this.setHighlightedCode(0)

    this.initHLJS()

    const support = clipboardCopy.isSupported()
    if (!support) {
      this.setState({ isClipboardSupported: support })
    }
  }

  initHLJS () {
    if (!window.hljs || !window.hljs.initHighlighting) {
      return setTimeout(this.initHLJS.bind(this), 300)
    }

    window.hljs.configure({ tabReplace: '  ' }) // 2 spaces
    window.hljs.initHighlighting()
  }

  alertCopied (success) {
    this.setState({
      copyBtnText: success ? 'Copied!' : 'Copy Unsuccessful'
    })

    setTimeout(this.revertCopiedText, 1000)
  }

  revertCopiedText () {
    this.setState({ copyBtnText: 'Copy Snippet' })
  }

  getFileExtension (lang) {
    switch (lang) {
      case 'Python': return 'py'
      case 'Java': return 'java'
      case 'C++': return 'cpp'
      case 'C#': return 'cs'
      case 'Node.JS': return 'js'
    }
  }

  getCurrentSnippet () {
    return this.props.snippets[this.state.activeTab].stages[this.state.activeStage]
  }

  setHighlightedCode (newStage, shouldAnimate) {
    this.minimaps.forEach((minimap, i) => {
      const last = minimap.querySelector('.minimap-section--active')

      if (last) {
        last.classList.remove('minimap-section--active')
      }

      const sections = minimap.querySelectorAll('.minimap-section')

      sections[newStage].classList.add('minimap-section--active')

      if (shouldAnimate === false) {
        newStage === 0
          ? minimap.scrollTop = 0
          : minimap.scrollTop = sections[newStage].offsetTop - 20
        return
      }

      return newStage === 0
        ? this.scrollToSection(minimap, 0)
        : this.scrollToSection(minimap, sections[newStage].offsetTop - 20)
    })
  }

  scrollToSection (minimap, dest) {
    if (dest === minimap.scrollTop) return

    let steps = 0
    const start = minimap.scrollTop
    const totalSteps = 30
    const diff = dest - start

    function tick () {
      steps += 1
      minimap.scrollTop = start + (diff * easing.easeOutQuart(steps / totalSteps))

      if (steps >= totalSteps) {
        minimap.scrollTop = dest
        return
      }

      window.requestAnimationFrame(tick)
    }

    window.requestAnimationFrame(tick)
  }

  onTabSelect (e) {
    const tab = Number(e.currentTarget.getAttribute('data-i'))

    if (tab === this.state.activeTab) return

    this.setState({ activeTab: tab }, () => {
      this.setHighlightedCode(this.state.activeStage, false)
    })
  }

  onDropdownSelect (newStage) {
    this.setState({ activeStage: newStage })
    this.setHighlightedCode(newStage)
  }

  onNavigateStage (e) {
    const direction = Number(e.currentTarget.getAttribute('data-direction'))
    const newStage = this.state.activeStage + direction

    if (newStage < 0 || newStage === this.props.snippets.length) return

    this.setState({ activeStage: newStage })
    this.setHighlightedCode(newStage)
  }

  onCopyBtnClick (e) {
    if (clipboardCopy(this.getCurrentSnippet())) {
      this.alertCopied(true)
    } else {
      this.alertCopied(false)
    }

    const cl = e.currentTarget.classList

    cl.add('animation--bounce-inwards')
    setTimeout(() => cl.remove('animation--bounce-inwards'), 600)
  }

  render () {
    return <div className='code-widget'>
      <div className='flex flex--wrap'>
        {this.props.snippets.map(this.renderTabBtn)}
      </div>

      <div className='code-widget__shadow'>
        {this.props.snippets.map(this.renderTabs)}
      </div>
    </div>
  }

  renderTabBtn (snippet, i) {
    return <button
      key={i}
      className={`code-widget__tab ${
        i === this.state.activeTab ? 'code-widget__tab--active' : ''}`}
      onClick={this.onTabSelect}
      data-i={i}>
      {snippet.language}
    </button>
  }

  renderTabs (snippet, i) {
    const stage = this.state.activeStage
    const tab = this.state.activeTab
    const downloadUrl = `/assets/files/what-is-mongodb.${
      this.getFileExtension(snippet.language)}`

    return <div
      key={i}
      style={tab === i ? null : { display: 'none' }}
      className='flex flex--wrap'>
      <div className='code-widget__column code-widget__column-1'>
        <div style={{ margin: '20px 20px 0' }}>
          <p><strong>Choose your step</strong></p>
          <Dropdown
            onSelect={this.onDropdownSelect}
            activeIndex={stage}
            options={this.stageTitles} />
        </div>

        <div className='code-widget__navigation'>
          <button
            className={`code-widget__navigation-btn ${
              stage === 0 ? '' : 'code-widget__navigation-btn--active'}`}
            onClick={this.onNavigateStage}
            data-direction='-1'>
            Back
          </button>

          <button
            className={`code-widget__navigation-btn ${
              stage === this.props.snippets.length - 1
                ? ''
                : 'code-widget__navigation-btn--active'}`}
            onClick={this.onNavigateStage}
            data-direction='1'>
            Next
          </button>
        </div>

        <div className='code-widget__progress'>
          <div className={`code-widget__progress-bar code-widget__progress-bar--stage-${stage}`} />
        </div>

        <p className='code-widget__description' dangerouslySetInnerHTML={{
          __html: this.props.descriptions[this.state.activeStage].content
        }} />

        <div className='code-widget__column-1-options'>
          {this.state.isClipboardSupported &&
          <button
            onClick={this.onCopyBtnClick}
            className='clipboard-btn btn-hollow btn-hollow--gray'>
            {this.state.copyBtnText}
          </button>}

          <div className='divider--horizontal' />

          <a href={snippet.documentation[stage]}>
            <p>
              <strong>Go to Documentation</strong>
              <svg width='20' height='10' viewBox='0 0 20 10' style={{ marginLeft: '7px' }}>
                <path
                  fill='#589636'
                  fillRule='evenodd'
                  d='M14.707.586L13.293 2l2.293 2.293H0v2h15.586l-2.293 2.293L14.707 10l4.707-4.707z' />
              </svg>
            </p>
          </a>
        </div>
      </div>

      <div className='code-widget__column code-widget__column-2'>
        {snippet.stages.map(function (code, j) {
          return <pre key={`${i}.${j}`} style={{ display: j === stage ? 'inherit' : 'none' }}>
            <code className={snippet.language.toLowerCase()}>{code.trim()}</code>
          </pre>
        })}
      </div>

      <div className='code-widget__column code-widget__column-3'>
        <pre>
          <code
            ref={(el) => { if (el) this.minimaps.push(el) }}
            className={snippet.language.toLowerCase()}>
            {snippet.sourceCode}
          </code>
        </pre>

        <a href={downloadUrl} download={downloadUrl}>
          <button className='download-file-btn btn-hollow btn-hollow--gray'>
            Download File
          </button>
        </a>
      </div>
    </div>
  }

  renderMinimap (code, language, numStages) {
    const commentToken = language === 'python' ? '#' : '//'

    let result = code

    for (let i = 1, l = numStages + 2; i < l; ++i) {
      const startIndex = result.indexOf(`${commentToken} ${i}.`)

      if (i < l - 1) {
        result = this.insert(
          result,
          `<span class='minimap-section' id='minimap-${i}'>`,
          startIndex)
      }

      if (i < 3) continue

      const endIndex = result.indexOf(
        `<span class='minimap-section' id='minimap-${i - 1}'>`) - (commentToken.length)

      result = this.insert(result, '</span>', endIndex)

      if (i === l - 1) result += '</span>'
    }

    return result
  }

  insert (source, text, index) {
    return `${source.slice(0, index)}${text}${source.slice(index)}`
  }
}

module.exports = CodeWidget

==> mongodb-ui-components-4.0.4.tgz: package/src/code-widget/index.js <==
const React = require('react')
const Dropdown = require('../dropdown')
const clipboardCopy = require('../shared/clipboard-copy')
const easing = require('../shared/easing')
const style = require('../shared/style')
const script = require('../shared/script')
const widgetStyle = require('./style')

class CodeWidget extends React.PureComponent {
  constructor (props) {
    super(props)

    this.state = {
      activeTab: 0,
      activeStage: 0,
      isClipboardSupported: true,
      copyBtnText: 'Copy Snippet'
    }

    this.minimaps = []
    this.stageTitles = props.descriptions.map((item) => item.title)

    this.onTabSelect = this.onTabSelect.bind(this)
    this.onNavigateStage = this.onNavigateStage.bind(this)
    this.onDropdownSelect = this.onDropdownSelect.bind(this)
    this.onCopyBtnClick = this.onCopyBtnClick.bind(this)
    this.revertCopiedText = this.revertCopiedText.bind(this)
    this.renderTabBtn = this.renderTabBtn.bind(this)
    this.renderTabs = this.renderTabs.bind(this)
  }

  componentDidMount () {
    style('code-widget', widgetStyle())
    script('script', 'https://cdnjs.cloudflare.com/ajax/libs/highlight.js/9.10.0/highlight.min.js')
    script('link', 'https://cdnjs.cloudflare.com/ajax/libs/highlight.js/9.10.0/styles/default.min.css')

    this.minimaps.forEach((minimap, i) => {
      minimap.innerHTML = this.renderMinimap(
        minimap.innerHTML, minimap.className, this.stageTitles.length)
    })

    this.setHighlightedCode(0)

    this.initHLJS()

    const support = clipboardCopy.isSupported()
    if (!support) {
      this.setState({ isClipboardSupported: support })
    }
  }

  initHLJS () {
    if (!window.hljs || !window.hljs.initHighlighting) {
      return setTimeout(this.initHLJS.bind(this), 300)
    }

    window.hljs.configure({ tabReplace: '  ' }) // 2 spaces
    window.hljs.initHighlighting()
  }

  alertCopied (success) {
    this.setState({
      copyBtnText: success ? 'Copied!' : 'Copy Unsuccessful'
    })

    setTimeout(this.revertCopiedText, 1000)
  }

  revertCopiedText () {
    this.setState({ copyBtnText: 'Copy Snippet' })
  }

  getFileExtension (lang) {
    switch (lang) {
      case 'Python': return 'py'
      case 'Java': return 'java'
      case 'C++': return 'cpp'
      case 'C#': return 'cs'
      case 'Node.JS': return 'js'
    }
  }

  getCurrentSnippet () {
    return this.props.snippets[this.state.activeTab].stages[this.state.activeStage]
  }

  setHighlightedCode (newStage, shouldAnimate) {
    this.minimaps.forEach((minimap, i) => {
      const last = minimap.querySelector('.minimap-section--active')

      if (last) {
        last.classList.remove('minimap-section--active')
      }

      const sections = minimap.querySelectorAll('.minimap-section')

      sections[newStage].classList.add('minimap-section--active')

      if (shouldAnimate === false) {
        newStage === 0
          ? minimap.scrollTop = 0
          : minimap.scrollTop = sections[newStage].offsetTop - 20
        return
      }

      return newStage === 0
        ? this.scrollToSection(minimap, 0)
        : this.scrollToSection(minimap, sections[newStage].offsetTop - 20)
    })
  }

  scrollToSection (minimap, dest) {
    if (dest === minimap.scrollTop) return

    let steps = 0
    const start = minimap.scrollTop
    const totalSteps = 30
    const diff = dest - start

    function tick () {
      steps += 1
      minimap.scrollTop = start + (diff * easing.easeOutQuart(steps / totalSteps))

      if (steps >= totalSteps) {
        minimap.scrollTop = dest
        return
      }

      window.requestAnimationFrame(tick)
    }

    window.requestAnimationFrame(tick)
  }

  onTabSelect (e) {
    const tab = Number(e.currentTarget.getAttribute('data-i'))

    if (tab === this.state.activeTab) return

    this.setState({ activeTab: tab }, () => {
      this.setHighlightedCode(this.state.activeStage, false)
    })
  }

  onDropdownSelect (newStage) {
    this.setState({ activeStage: newStage })
    this.setHighlightedCode(newStage)
  }

  onNavigateStage (e) {
    const direction = Number(e.currentTarget.getAttribute('data-direction'))
    const newStage = this.state.activeStage + direction

    if (newStage < 0 || newStage === this.props.snippets.length) return

    this.setState({ activeStage: newStage })
    this.setHighlightedCode(newStage)
  }

  onCopyBtnClick (e) {
    if (clipboardCopy(this.getCurrentSnippet())) {
      this.alertCopied(true)
    } else {
      this.alertCopied(false)
    }

    const cl = e.currentTarget.classList

    cl.add('animation--bounce-inwards')
    setTimeout(() => cl.remove('animation--bounce-inwards'), 600)
  }

  render () {
    return <div className='code-widget'>
      <div className='flex flex--wrap'>
        {this.props.snippets.map(this.renderTabBtn)}
      </div>

      <div className='code-widget__shadow'>
        {this.props.snippets.map(this.renderTabs)}
      </div>
    </div>
  }

  renderTabBtn (snippet, i) {
    return <button
      key={i}
      className={`code-widget__tab ${
        i === this.state.activeTab ? 'code-widget__tab--active' : ''}`}
      onClick={this.onTabSelect}
      data-i={i}>
      {snippet.language}
    </button>
  }

  renderTabs (snippet, i) {
    const stage = this.state.activeStage
    const tab = this.state.activeTab
    const downloadUrl = snippet.sourceUrl || `/assets/files/what-is-mongodb.${
      this.getFileExtension(snippet.language)}`

    return <div
      key={i}
      style={tab === i ? null : { display: 'none' }}
      className='flex flex--wrap'>
      <div className='code-widget__column code-widget__column-1'>
        <div style={{ margin: '20px 20px 0' }}>
          <p><strong>Choose your step</strong></p>
          <Dropdown
            onSelect={this.onDropdownSelect}
            activeIndex={stage}
            options={this.stageTitles} />
        </div>

        <div className='code-widget__navigation'>
          <button
            className={`code-widget__navigation-btn ${
              stage === 0 ? '' : 'code-widget__navigation-btn--active'}`}
            onClick={this.onNavigateStage}
            data-direction='-1'>
            Back
          </button>

          <button
            className={`code-widget__navigation-btn ${
              stage === this.props.snippets.length - 1
                ? ''
                : 'code-widget__navigation-btn--active'}`}
            onClick={this.onNavigateStage}
            data-direction='1'>
            Next
          </button>
        </div>

        <div className='code-widget__progress'>
          <div className={`code-widget__progress-bar code-widget__progress-bar--stage-${stage}`} />
        </div>

        <p className='code-widget__description' dangerouslySetInnerHTML={{
          __html: this.props.descriptions[this.state.activeStage].content
        }} />

        <div className='code-widget__column-1-options'>
          {this.state.isClipboardSupported &&
          <button
            onClick={this.onCopyBtnClick}
            className='clipboard-btn btn-hollow btn-hollow--gray'>
            {this.state.copyBtnText}
          </button>}

          <div className='divider--horizontal' />

          <a href={snippet.documentation[stage]}>
            <p>
              <strong>Go to Documentation</strong>
              <svg width='20' height='10' viewBox='0 0 20 10' style={{ marginLeft: '7px' }}>
                <path
                  fill='#589636'
                  fillRule='evenodd'
                  d='M14.707.586L13.293 2l2.293 2.293H0v2h15.586l-2.293 2.293L14.707 10l4.707-4.707z' />
              </svg>
            </p>
          </a>
        </div>
      </div>

      <div className='code-widget__column code-widget__column-2'>
        {snippet.stages.map(function (code, j) {
          return <pre key={`${i}.${j}`} style={{ display: j === stage ? 'inherit' : 'none' }}>
            <code className={snippet.language.toLowerCase()}>{code.trim()}</code>
          </pre>
        })}
      </div>

      <div className='code-widget__column code-widget__column-3'>
        <pre>
          <code
            ref={(el) => { if (el) this.minimaps.push(el) }}
            className={snippet.language.toLowerCase()}>
            {snippet.sourceCode}
          </code>
        </pre>

        <a href={downloadUrl} download={downloadUrl}>
          <button className='download-file-btn btn-hollow btn-hollow--gray'>
            Download File
          </button>
        </a>
      </div>
    </div>
  }

  renderMinimap (code, language, numStages) {
    const commentToken = language === 'python' ? '#' : '//'

    let result = code

    for (let i = 1, l = numStages + 2; i < l; ++i) {
      const startIndex = result.indexOf(`${commentToken} ${i}.`)

      if (i < l - 1) {
        result = this.insert(
          result,
          `<span class='minimap-section' id='minimap-${i}'>`,
          startIndex)
      }

      if (i < 3) continue

      const endIndex = result.indexOf(
        `<span class='minimap-section' id='minimap-${i - 1}'>`) - (commentToken.length)

      result = this.insert(result, '</span>', endIndex)

      if (i === l - 1) result += '</span>'
    }

    return result
  }

  insert (source, text, index) {
    return `${source.slice(0, index)}${text}${source.slice(index)}`
  }
}

module.exports = CodeWidget
